const alphabet = 'abcdefghijklmnopqrstuvwxyz234567'

function decode_chunk(input, in_offset, output, out_offset) {
  let b = BigInt(0);
  for (let i = 0; i < 8; i++) {
    b = (b << 5n) | BigInt(alphabet.indexOf(input[in_offset + i]));
  }
  for (let j = 4; j >= 0; j--) {
    output[out_offset + 4 - j] = Number(b >> BigInt(j * 8));
  }
}

function decode(input) {
  const out_len = Math.ceil((input.length / 8) * 5);
  const output = Buffer.alloc(out_len);

  const q = Math.floor(input.length / 8);
  for (let i = 0; i < q; i++) {
    decode_chunk(input, i * 8, output, i * 5);
  }

  return output;
}
exports.decode = decode;

function encode_chunk(input, in_offset, output, out_offset) {
  let b = BigInt(0);
  for (let i = 0; i < 5; i++) {
    b = (b << 8n) | BigInt(input[in_offset + i]);
  }
  for (let i = 7; i >= 0; i--) {
    b = b << BigInt(24 + (7 - i) * 5);
    b = b >> BigInt(24 + (7 - i) * 5);

    const c = Number(b >> BigInt(i * 5)) % 32;
    output[out_offset + 7 - i] = alphabet.charCodeAt(c);
  }
}

function encode(input) {
  const out_len = Math.ceil(input.length / 5 * 8);
  const output = Buffer.alloc(out_len);

  const q = Math.floor(input.length / 5);
  for (let i = 0; i < q; i++) {
    encode_chunk(input, i * 5, output, i * 8);
  }

  return output.toString('ascii');
}
exports.encode = encode;